/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include <cassert>
#include <cstring>
#include <memory>
#include <string>
#include <vector>

#include "common/common.h"
#include "parser/parser.h"
#include "system/sm.h"
#include "system/sm_meta.h"

class Query {
 public:
  std::shared_ptr<ast::TreeNode> parse;
  // TODO jointree
  // where条件
  std::vector<Condition> conds;
  // 投影列
  std::vector<TabCol> cols;
  // 表名
  std::vector<std::string> tables;
  // update 的set 值
  std::vector<SetClause> set_clauses;
  // insert 的values值
  std::vector<Value> values;

  // group列
  std::vector<TabCol> group_cols;
  // having条件
  std::vector<Condition> having_conds;

  //sort
  TabCol sort_col;

  Query() {}
};

class Analyze {
 private:
  SmManager *sm_manager_;

 public:
  Analyze(SmManager *sm_manager) : sm_manager_(sm_manager) {}
  ~Analyze() {}

  std::shared_ptr<Query> do_analyze(std::shared_ptr<ast::TreeNode> root);

 private:
  void check_aggregation_type(std::vector<TabCol> &cols, std::vector<ColMeta> &col_metas);
  void check_group_list(std::vector<TabCol> &query_cols, std::vector<TabCol> &group_cols,
                        std::vector<ColMeta> &all_cols);
  void check_having_conds(std::vector<TabCol> &query_cols, std::vector<TabCol> &group_cols,
                          std::vector<Condition> &having_conds, std::vector<ColMeta> &all_cols);
  TabCol check_column(const std::vector<ColMeta> &all_cols, TabCol target);
  void get_all_cols(const std::vector<std::string> &tab_names, std::vector<ColMeta> &all_cols);
  void get_clause(const std::vector<std::shared_ptr<ast::BinaryExpr>> &sv_conds,
                  std::vector<Condition> &conds, bool is_where = true);
  void check_clause(const std::vector<std::string> &tab_names, std::vector<Condition> &conds);
  Value convert_sv_value(const std::shared_ptr<ast::Value> &sv_val);
  CompOp convert_sv_comp_op(ast::SvCompOp op);
};
