/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once
#include <cstddef>
#include <memory>

#include "common/common.h"
#include "execution_defs.h"
#include "execution_manager.h"
#include "executor_abstract.h"
#include "index/ix.h"
#include "record/rm_defs.h"
#include "system/sm.h"
#include "system/sm_meta.h"

class SortExecutor : public AbstractExecutor {
 private:
  std::unique_ptr<AbstractExecutor> prev_;
  ColMeta cols_;
  bool is_desc_;
  std::vector<std::unique_ptr<RmRecord>> tuples_;
  std::vector<std::unique_ptr<RmRecord>>::iterator current_tuple_;

 public:
  SortExecutor(std::unique_ptr<AbstractExecutor> prev, TabCol sel_cols, bool is_desc) {
    prev_ = std::move(prev);
    cols_ = *get_col(prev_->cols(), sel_cols);
    is_desc_ = is_desc;
  }

  void beginTuple() override {
    for (prev_->beginTuple(); !prev_->is_end(); prev_->nextTuple()) {
      auto tuple = prev_->Next();
      tuples_.emplace_back(std::move(tuple));
    }

    Value lhs;
    Value rhs;
    lhs.type = cols_.type;
    rhs.type = cols_.type;

    auto cmp = [this, &lhs, &rhs](std::unique_ptr<RmRecord> &left,
                                  std::unique_ptr<RmRecord> &right) {
      lhs.set_data(left->data + cols_.offset, cols_.len);
      rhs.set_data(right->data + cols_.offset, cols_.len);

      CompResult res = lhs.compare_value(rhs);
      return (!is_desc_ ? res == CompResult::LT : res == CompResult::GT);
    };

    std::sort(tuples_.begin(), tuples_.end(), cmp);

    current_tuple_ = tuples_.begin();
  }

  void nextTuple() override { ++current_tuple_; }

  std::unique_ptr<RmRecord> Next() override { return std::make_unique<RmRecord>(**current_tuple_); }

  Rid &rid() override { return _abstract_rid; }
  bool is_end() const override { return current_tuple_ == tuples_.end(); }
  const std::vector<ColMeta> &cols() const override { return prev_->cols(); }
  size_t tupleLen() const override { return prev_->tupleLen(); }
};